# joylink  device sdk  java 版 用户手册  


##  1. 概述及说明  
为简化设备开发流程，实现设备的快速接入，提供了java版的设备接入SDK。 在使用SDK之前，默认用户已经在物联平台完成设备创建等相关工作，并得到了设备标识、设备证书等所需要的信息。 



##  2. 名词解释  

为了方便用户能够快速的对使用手册进行约定，进而进行设备接入，对涉及到的相关名词进行简单说明。

**直连设备**：具备通过MQTT协议连接到物联平台的设备； 

**非直连设备**：不具备通过MQTT协议连接到物联平台的设备； 

**网关设备**：具备通过MQTT协议连接到物联平台，并且能够为非直连设备提供代理接入物联平台； 

**子设备**：这里指的时非直连设备； 

**属性**：一些可能表示状态，条件的独立参数，例如时间，名称，版本，温度，容量，开关等基础属性； 

**方法**：一个方法是一个具体的操作，有入参与出参，通过调用可以得到一个结果； 

**事件**：一些由条件触发的消息，由设备端主动发出；

**实例名 （key）**：物模型中一个属性实例，事件实例，方法实例，模型实例的唯一标识。需要在物模型中保持唯一；

**服务实例名**：可选值 {NULL, "connection_agent", "ota", “config”}，SDK目前支持的取值 {NULL， “connection_agent”}；

**connection_agent**：一个内置的代理服务，对于网关设备此服务为必选，用于代理子设备完成相应的功能；直连设备可不需要该服务;  



##  3. 引用文件  

无 



##  4. 安装编译和运行

### 4.1. 最低要求
要使用SDK，开发者需要 Java 1.7+

### 4.2. 使用 Maven 安装
开发者在工程中使用 java 版SDK，推荐使用 Maven 进行安装，只需将以下依赖项添加到Maven项目的POM文件中：
```
<dependencies>
  <dependency>
    <groupId>com.joylink5</groupId>
    <artifactId>joylink5-device-sdk-java</artifactId>
    <version>1.3.7</version>
  </dependency>
</dependencies>
```
也可以使用以下依赖项定义来安装SDK附带的示例应用程序
```
<dependencies>
  <dependency>
    <groupId>com.joylink5</groupId>
    <artifactId>joylink5-device-sdk-java-samples</artifactId>
    <version>1.3.7</version>
  </dependency>
</dependencies>
```

### 4.3. 使用最新的JAR安装
开发者可以简单地将JAR文件提取并复制到项目的库目录中，然后更新IDE以将它们包括到库构建路径中。
您还需要添加两个SDK依赖的库：
[1]  Jackson 2.x, 包括 Jackson-core jackson-core 和 Jackson-databind jackson-databind
[2] Paho MQTT client for Java 1.1.x.

### 4.4. 源码编译
开发者可以从源代码构建SDK及其示例应用程序
```
$ cd joylink5_device_sdk_java
$ mvn clean install -Dgpg.skip=true
```
### 4.5. 运行示例应用程序
开发者可以使用如下的命令行运行示例应用程序
[1] 运行 DeviceSample 示例应用程序

```
$ mvn exec:java -pl joylink5-device-sdk-java-samples -Dexec.mainClass="com.joylink5.services.iot.client.sample.device.DeviceSample" -Dexec.args="-clientEndpoint <unique mqtthostname> -clientEndpointPort <unique mqttport> -deviceId <unique device> -caCertificateFile <rootCa path> -certificateFile <client crt path> -privateKeyFile <client key path>"
```


[2] 运行 GatewaySample 示例应用程序

```
Dexec.mainClass="com.joylink5.services.iot.client.sample.gateway.GatewaySample" -Dexec.args="-clientEndpoint <unique mqtthostname> -clientEndpointPort <unique mqttport> -deviceId <unique device> -caCertificateFile <rootCa path> -certificateFile <client crt path> -privateKeyFile <client key path>"
```

## 5. 用户开发说明
### 5.1. 初始化客户端
开发者使用SDK来进行应用的开发，必须先初始化 JoylinkMqttClient 客户端。初始化客户端，需要物管平台的hostname，端口号，设备标识和设备证书。
```
joylinkClient = new JoylinkMqttClient(clientEndpoint, deviceId, sslSocketFactory, clientEndpointPort);
```
### 5.2. 创建设备
设备的创建和初始化，需要根据在物管平台进行的物模型定义。设备类型主要有三种，直连设备、网关设备和子设备。其中网关设备本质上也属于直连设备。

#### 5.2.1. 直连设备
开发者可以通过继承 JoylinkDevice 类来实现直连设备，如果设备支持OTA功能，需要实现 JoylinkOtaFunctions 接口，如下所示：
```
public class ConnectedWindow extends JoylinkDevice implements JoylinkOtaFunctions {
	...
}
```


#### 5.2.2. 网关设备
网关设备本质是一种直连设备，与直连设备相比，网关设备含有connection_agent 服务模型，因此在实例化网关模型的时候，还需要实现 JoylinkConnectionAgentFunctions 接口。接口的具体说明以及实现示例会在后续章节中给出。
```
public class ConnectedWindow extends JoylinkDevice implements JoylinkOtaFunctions ,JoylinkConnectionAgentFunctions {
	...
}
```



##### 5.2.2.1 connect_agent 方法调用

连接代理服务内置了4个方法调用，用来接收物管平台下发的网络拓扑相关的指令，包括添加子设备、删除子设备、更新子设备，查询子设备。
如果网关设备选择自行维护代理的子设备信息，可以不实现该方法调用。

```
public interface JoylinkConnectionAgentFunctions {

    String onFuncAddDevices(String inJson);

    String onFuncDeleteDevices(String inJson);

    String onFuncUpdateDevices(String inJson);

    String onFuncGetDevices();

}
```

下面对每个接口的功能进行详细说明：



> >**onFuncAddDevices**

用于处理物管平台下发的添加子设备的指令，入参为子设备信息数组。

实现的示例代码：

```
    @Override
    public String onFuncAddDevices(String inJson) {
        // parse devices
        ObjectMapper mapper = new ObjectMapper();
        JsonNode rootNode;
        try {
            rootNode = mapper.readTree(inJson);
            if (rootNode == null) {
                System.out.println("Parse inJson is null");
                super.setErrorCode(JoylinkDeviceErrorCode.BAD_REQUEST);
                return "";
            }
            JsonNode node = rootNode.get("devices");
            if (node == null) {
                System.out.println("function call in parameter devices is null");
                super.setErrorCode(JoylinkDeviceErrorCode.BAD_REQUEST);
                return "";
            }

            for (JsonNode item : node) {
                String subDevInfo = item.toString();
                String subDevId = item.get("device-id").asText();
                subDevices.put(subDevId, subDevInfo);
            }

            return "";

        } catch (IOException e) {
            System.out.println("Process function call failed");
            super.setErrorCode(JoylinkDeviceErrorCode.BAD_REQUEST);
            return "";
        }
    }
```



> >**onFuncDeleteDevices**

用于处理物管平台下发的删除子设备的指令，入参为需要删除的子设备信息数组。
实现的示例代码如下：

```
    @Override
    public String onFuncDeleteDevices(String inJson) {
        ObjectMapper mapper = new ObjectMapper();
        JsonNode rootNode;
        try {
            rootNode = mapper.readTree(inJson);
            if (rootNode == null) {
                System.out.println("Parse inJson is null");
                super.setErrorCode(JoylinkDeviceErrorCode.BAD_REQUEST);
                return "";
            }
            JsonNode node = rootNode.get("devices");
            if (node == null) {
                System.out.println("function call in parameter devices is null");
                super.setErrorCode(JoylinkDeviceErrorCode.BAD_REQUEST);
                return "";
            }

            for (JsonNode item : node) {
                String subDevId = item.get("device-id").asText();
                subDevices.remove(subDevId);
            }

            return "";

        } catch (IOException e) {
            System.out.println("Process function call failed");
            super.setErrorCode(JoylinkDeviceErrorCode.BAD_REQUEST);
            return "";
        }
    }
```



> > **onFuncUpdateDevices**

用于处理物管平台下发的更新子设备的指令，入参为需要更新的子设备信息数组。
实现示例代码如下：

```
    @Override
    public String onFuncUpdateDevices(String inJson) {
        subDevices.clear();
        return onFuncAddDevices(inJson);
    }
```



> > **onFuncGetDevices**

用于处理物管平台下发的获取子设备信息的指令，出参为当前的子设备信息数组。
实现示例代码如下：

```
    @Override
    public String onFuncGetDevices() {
        try {
            ObjectMapper mapper = new ObjectMapper();
            List<Object> devices = new ArrayList<>();
            for (String key : subDevices.keySet()) {
                JsonNode node = mapper.readTree(subDevices.get(key));
                devices.add(node);
            }
            return mapper.writeValueAsString(devices);
        } catch (IOException e) {
            super.setErrorCode(JoylinkDeviceErrorCode.BAD_REQUEST);
            return "";
        }
    }
```





#### 5.2.3. 子设备

开发者可以通过继承 JoylinkSubDevice 来实例化一个子设备。
```
public class SubWindow extends JoylinkSubDevice {
	...
}
```

#### 5.2.4. 设备属性
设备属性是通过 Java的 annotation 注解来实现。需要开发者根据实际的属性类型来实现相应的get和set方法，下面以一个boolean型的属性为例：
```
    @JoylinkDeviceProperty (name = "window.windowOpen")
    private boolean windowOpen;

    public boolean getWindowOpen() {
        // 1. read the window state from the window actuator
        boolean reportedState = this.windowOpen;
        System.out.println(
                System.currentTimeMillis() + " >>> reported window_windowOpen state: " + (reportedState ? "open" : "closed"));

        
        // 2. return the current window state
        return reportedState;
    }

    public void setWindowOpen(boolean desiredState) {
        // 1. update the window actuator with the desired state
        System.out.println("window.windowOpen should be set:" + String.valueOf(desiredState));
        this.windowOpen = desiredState;

        System.out.println(
                System.currentTimeMillis() + " <<< desired window state to " + (this.windowOpen ? "open" : "closed"));
    }
```
其中 name 为在物管平台定义的物模型中属性的key。



#### 5.2.5. 方法调用

设备的方法调用也是通过 Java的 annotation 注解来实现，需要开发者实现get和set方法，其中get方法的返回类型为Json字符串，即方法调用完成后的出参。set方法的入参类型为字符串，即方法调用的入参。具体示例如下：
```
    @JoylinkDeviceFunction (name = "window.open")
    private String openWindow;
    public String getOpenWindow() {
        return this.openWindow;
    }

    public void setOpenWindow(String inParam) {
        System.out.println("Execute Function Call window.open With Parametres:" + inParam);
        this.openWindow = "{\"message\":\"success\"}";
    }
```



#### 5.2.6. OTA 方法调用接口

如果设备要支持ota，实现ota服务模型内置的方法调用接口。
```
public interface JoylinkOtaFunctions {

    void onFuncOta(String argsJson);

    void onFuncCancelTask(String taskId);

    String onFuncQueryState(String taskId);

    void onFuncExecTask(String taskId);

}
```
下面具体对每个接口函数的功能进行一下说明：



> > **onFuncOta**

```
void onFuncOta(String argsJson);
```
该函数用于接收物管平台下发的ota升级指令，以及相应的ota参数，包括任务标识、升级包，版本号，升级包的url、安装方式等参数，开发者需要根据实际情况及相应的参数执行ota升级过程。

示例实现如下：
```
    @Override
    public void onFuncOta(String argsJson) {
        System.out.println("\r\n\r\nHello World From onFuncOta \r\n\r\n");

        // get task-id url version
        ObjectMapper mapper = new ObjectMapper();
        JsonNode rootNode;
        try {
            rootNode = mapper.readTree(argsJson);
            this.otaTaskId = rootNode.get("task-id").asText();
            this.otaVersion = rootNode.get("version").asText();
            this.otaUrl = rootNode.get("url").asText();
            this.otaPackageName = rootNode.get("package-name").asText();
        } catch (IOException e) {
            System.out.println("Parse OTA Function Call ota.ota in parameter failed");
            super.setErrorCode(JoylinkDeviceErrorCode.BAD_REQUEST);
            return;
        }

        // execute thread for ota task
        otaTaskThread = new Thread(){
            @Override
            public void run() {
                super.run();
                System.out.println("Start ota task:" + otaTaskId + " procedure...");
                // Intialise State
                otaProgress = 0;
                otaErrorCode = OtaErrorCode.NONE.getValue();
                otaState = OtaState.PREPARE.getValue();

                try {
                    Thread.sleep(2000);

                    // send state change report
                    sendOtaEventStateChanged(otaTaskId, otaState, otaErrorCode, otaProgress);
                    Thread.sleep(1000);

                    // according to install-mode, decided if should interactive with user

                    // start to download package according to url
                    // do download procedure
                    otaState = OtaState.DOWNLOADING.getValue();
                    otaProgress = 1;
                    System.out.println("\r\nDownloading package from " + otaUrl + "\r\n");
                    sendOtaEventStateChanged(otaTaskId, otaState, otaErrorCode, otaProgress);
                    Thread.sleep(1000);

                    // when download done, send state changed
                    otaState = OtaState.DOWNLOAD_DONE.getValue();
                    otaProgress = 10;
                    sendOtaEventStateChanged(otaTaskId, otaState, otaErrorCode, otaProgress);
                    Thread.sleep(1000);

                    // start intall procedure
                    otaState = OtaState.INSTALLING.getValue();
                    otaProgress = 10;
                    sendOtaEventStateChanged(otaTaskId, otaState, otaErrorCode, otaProgress);
                    Thread.sleep(1000);

                    // 
                    otaProgress = 80;
                    sendOtaEventStateChanged(otaTaskId, otaState, otaErrorCode, otaProgress);
                    Thread.sleep(1000);

                    // Install Completed
                    otaState = OtaState.SUCCESS.getValue();
                    otaProgress = 100;
                    sendOtaEventStateChanged(otaTaskId, otaState, otaErrorCode, otaProgress);
                    Thread.sleep(100);

                    // send Version changed
                    otaCurrentVersion = otaVersion;
                    sendOtaEventVersionChanged(otaCurrentVersion);

                } catch (InterruptedException e) {
                    System.out.println("\r\n\r\nOta Thread Recv interrupt\r\n\r\n");
                }
            }
        };
        otaTaskThread.start();

    }
```



> > **onFuncCancelTask**

```
void onFuncCancelTask(String taskId);
```
该函数用于接收物管平台下发的取消升级指令，入参为任务标识，开发者需要根据要求停止正在进行的升级任务。

示例实现代码如下：
```
    @Override
    public void onFuncCancelTask(String taskId) {
        System.out.println("\r\n\r\nCancel OTA Task:" + taskId + "\r\n\r\n\r\n");
        if (otaTaskThread.isAlive()) {
            otaTaskThread.interrupt();
            otaState = OtaState.FAILED.getValue();
            otaErrorCode = OtaErrorCode.TASK_CANCEL.getValue();
        }
    }
```



> > **onFuncQueryState**

```
String onFuncQueryState(String taskId);
```
该函数用于接收物管平台下发的状态查询指令，入参为任务标识，开发者需要将任务当前的状态进行返回。返回的字符串可以通过 
```
onOtaBuildState(otaTaskId, otaState, otaErrorCode, otaProgress);
```
方法生成，其中入参为任务标识，当前状态，错误码，当前进度百分比。

示例实现代码：
```
    @Override
    public String onFuncQueryState(String taskId) {
        System.out.println("\r\n\r\nQuery State OTA Task:" + taskId + "\r\n\r\n\r\n");
        return super.onOtaBuildState(otaTaskId, otaState, otaErrorCode, otaProgress);
    }
```



> >**onFuncExecTask**

```
void onFuncExecTask(String taskId);
```
该函数用于接收物管平台下发的任务执行执行，入参为任务标识。
目前该功能为保留功能。



### 5.3. 应用逻辑流程开发说明

开发者根据实际的应用场景及物模型定义完成了设备的创建之后，就可以开始进行应用的开发，一下按照直连设备和网关代理设备两个应用场景对涉及到的协议交互以及实现方式进行说明。



#### 5.3.1. 直连设备场景

##### 5.3.1.1. 客户端绑定设备

完成设备创建之后，需要将设备与客户端进行绑定：

```
// 实例化设备 其中 deviceId 为设备标识 在物管平台注册设备时获得
ConnectedWindow connectedWindow = new ConnectedWindow(deviceId);

// 设置设备的物模型信息 该信息可在物管平台创建完成物类型后通过导出物模型获得
connectedWindow.setThingModelInfo("urn:user-spec-v1:thing-model:001:9545fe83892112f08bf4cfadf8f51890", "v1.0.0");

// 绑定设备到客户端
joylinkClient.attach(connectedWindow);
```



##### 5.3.1.2. 客户端与物管平台建立连接

在完成设备绑定后，可通过如下方式建立与物管平台的连接：

```
// 建立与物管平台的连接
joylinkClient.connect();
```



调用该接口后，开发者可以通过如下接口获取与物管平台的连接状态，来进行相关应用逻辑的处理：

```
newStatus = joylinkClient.getConnectionStatus();
```

客户端与物管平台的连接完成建立之后，包括初次连接和断网重连（SDK默认支持自动重连机制）的情况，设备需要向物管平台进行设备物模型信息、设备属性的上报、OTA当前版本号的上报（如果设备支持OTA功能）。


##### 5.3.1.3. 设备物模型上报
完成与物管平台的连接后，设备可以通过如下方式物模行进行上报：
```
connectedWindow.sendThingModelPost();
```
物管平台会对上报的物模型信息进行校验，上报的响应SDK目前按照日志的形式打印出来。

##### 5.3.1.4. 设备属性上报
设备属性的上报可通过如下方式实现：
```
propJson = connectedWindow.onPropertiesPost();
connectedWindow.sendPropertiesPost(connectedWindow.getLocalVersion().get(), propJson);
```

其中：
```
connectedWindow.getLocalVersion().get()
```
为获取当前设备属性的版本号。

协议约定，当设备的属性出现变化时需要进行属性上报，如果属性中包含**可写属性**，需要将属性版本号增加，版本号增加可通过如下方式实现：
```
connectedWindow.setLocalVersion(newVersion)
```

##### 5.3.1.5. 设备事件上报
设备端出现事件时，可通过如下方式进行事件上报：
```
String evtKey = "window.alter";
String paraJson = "{\"message\":\"danger, danger,danger!!\"}";
connectedWindow.sendEventPost(evtKey, paraJson);
```
其中入参为物模型定义的事件key和相应的参数json文本。

##### 5.3.1.6. 设备OTA当前版本号上报
设备当前OTA版本号的上报可通过如下方式实现：
```
connectedWindow.sendOtaEventVersionChanged(connectedWindow.getOtaCurrentVersion());
```



#### 5.3.2.  网关应用场景

##### 5.3.2.1. 网关设备处理说明
因为网关设备本质上也是一种直连设备，因此网关设备自身的一些处理逻辑可参考直连设备的处理。这里不再进行重复罗列。
**开发者的实际应用场景中，如果不关心网关设备自身的功能，开发者可以不用实例化网关设备，但是需要在物管平台进行网关设备注册，创建客户端时使用设备标识和证书为网关设备的设备标识和证书**

##### 5.3.2.2. 客户端与子设备的绑定

客户端与子设备的绑定可通过如下方式实现：
```
// 实例化子设备 deviceId 为子设备的设备标识
SubWindow subWindow = new SubWindow(deviceId);

// 设置子设备的信息 在物管平台创建设备时 可以获得该信息
subWindow.setSubDevInfo("java_sdk_subdevice", "10106e45334b300", "", "urn:user-spec-v1:thing-model:003:b8fd0d04580df40839663b02ea810147", "", "AABBCCDD", "tcp", "");

// 客户端与子设备绑定
joylinkClient.attach(subWindow);
```
**附加说明**
> 如果有多个子设备，使用同样的方式建立与客户端的绑定关系。

##### 5.3.2.3. 客户端与物管平台连接
客户端与物管平台可以通过如下关系建立连接：
```
// 建立与物管平台的连接
joylinkClient.connect();
```
调用该接口后，开发者可以通过如下接口获取与物管平台的连接状态，来进行相关应用逻辑的处理：

```
newStatus = joylinkClient.getConnectionStatus();
```

连接建立完成后，设备端需要做如下处理：
[1] 上报子设备添加设备事件；
[2] 上报子设备上线事件;
[3] 上报子设备当前OTA版本（如果支持OTA功能）；
[4] 上报子设备当前属性；

##### 5.3.2.4. 子设备上报添加设备事件
子设备可以通过网关设备的connect_agent服务模型向物管平台上报自身与网关设备的拓扑关系：
```
subWindow.sendEventAddDevices();
```

##### 5.3.2.5. 子设备上报在线状态
子设备可以通过网关设备的connect_agent服务模型向物管平台上报自身的在线状态：
```
// 在线
subWindow.sendDeviceOnlineState(true);

// 离线
subWindow.sendDeviceOnlineState(false);
```

##### 5.3.2.6. 子设备上报属性
子设备可以通过网关设备的connect_agent服务模型向物管平台上报属性：
```
propJson = subWindow.onPropertiesPost();
subWindow.sendPropertiesPost(subWindow.getLocalVersion().get(), propJson);
```

##### 5.3.2.7. 子设备上报事件
子设备可以通过网关设备的connect_agent服务模型向物管平台上报事件：
```
String evtKey = "subwindow.alter";
String paraJson = "{\"message\":\"alter, alter, alter!!\"}";
subWindow.sendEventPost(evtKey, paraJson);
```



-------------文档结束-------------

* * *
