/*
 * Copyright 2020 JD.com, Inc. or its affiliates. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License").
 * You may not use this file except in compliance with the License.
 * A copy of the License is located at
 *
 *  https://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * or in the "license" file accompanying this file. This file is distributed
 * on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

package com.joylink5.services.iot.client.sample.gateway;

import java.util.Random;

import com.joylink5.services.iot.client.JoylinkSubDevice;
import com.joylink5.services.iot.client.JoylinkDeviceFunction;
import com.joylink5.services.iot.client.JoylinkDeviceProperty;


/**
 * This class encapsulates an actual device. It extends {@link JoylinkDevice} to
 * define properties that are to be kept in sync with device shadow.
 */
public class SubWindow extends JoylinkSubDevice {

    public SubWindow(String deviceId) {
        super(deviceId);
    }

    @JoylinkDeviceProperty (name = "subwindow.roomTemperature")
    private float roomTemperature;

    @JoylinkDeviceProperty (name = "subwindow.windowOpen")
    private boolean windowOpen;

    public boolean getWindowOpen() {
        // 1. read the window state from the window actuator
        boolean reportedState = this.windowOpen;
        System.out.println(
                System.currentTimeMillis() + " >>> reported window_windowOpen state: " + (reportedState ? "open" : "closed"));

        
        // 2. return the current window state
        return reportedState;
    }

    public void setWindowOpen(boolean desiredState) {
        // 1. update the window actuator with the desired state
        System.out.println("window.windowOpen should be set:" + String.valueOf(desiredState));
        this.windowOpen = desiredState;

        System.out.println(
                System.currentTimeMillis() + " <<< desired window state to " + (this.windowOpen ? "open" : "closed"));
    }

    public float getRoomTemperature() {
        // 1. Read the actual room temperature from the thermostat
        Random rand = new Random();
        float minTemperature = 20.0f;
        float maxTemperature = 85.0f;
        float reportedTemperature = rand.nextFloat() * (maxTemperature - minTemperature) + minTemperature;

        // 2. (optionally) update the local copy
        this.roomTemperature = reportedTemperature;

        // 3. return the current room temperature
        System.out.println(System.currentTimeMillis() + " >>> reported room temperature: " + reportedTemperature);
        return this.roomTemperature;
    }

    public void setRoomTemperature(float desiredTemperature) {
        // no-op as room temperature is a read-only property. It's not required
        // to have this setter.
    }

    @JoylinkDeviceFunction (name = "subwindow.open")
    private String openWindow;
    public String getOpenWindow() {
        return this.openWindow;
    }

    public void setOpenWindow(String inParam) {
        System.out.println("Execute Function Call window.open With Parametres:" + inParam);
        this.openWindow = "{\"message\":\"success\"}";
    }

}
