/*
 * Copyright 2020 JD.com, Inc. or its affiliates. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License").
 * You may not use this file except in compliance with the License.
 * A copy of the License is located at
 *
 *  https://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * or in the "license" file accompanying this file. This file is distributed
 * on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

package com.joylink5.services.iot.client;

/**
 * The class provides default values for the library. All the values defined
 * here can be overridden at runtime through setter functions in
 * {@link JoylinkMqttClient} and {@link JoylinkDevice}.
 */
public class JoylinkConfig {

    /**
     * The default value for number of client threads. See also
     * {@link JoylinkMqttClient#getNumOfClientThreads()}.
     */
    public static final int NUM_OF_CLIENT_THREADS = 1;

    /**
     * The default value for client connection timeout (milliseconds). See also
     * {@link JoylinkMqttClient#getConnectionTimeout()}.
     */
    public static final int CONNECTION_TIMEOUT = 30000;

    /**
     * The default value for service acknowledge timeout (milliseconds). See
     * also {@link JoylinkMqttClient#getServerAckTimeout()}.
     */
    public static final int SERVER_ACK_TIMEOUT = 3000;

    /**
     * The default value for client keep-alive interval (milliseconds). See also
     * {@link JoylinkMqttClient#getKeepAliveInterval()}.
     */
    public static final int KEEP_ALIVE_INTERVAL = 600000;

    /**
     * The default value for maximum connection retry times. See also
     * {@link JoylinkMqttClient#getMaxConnectionRetries()}.
     */
    public static final int MAX_CONNECTION_RETRIES = 5;

    /**
     * The default value for connection base retry delay (milliseconds). See
     * also {@link JoylinkMqttClient#getBaseRetryDelay()}.
     */
    public static final int CONNECTION_BASE_RETRY_DELAY = 3000;

    /**
     * The default value for connection maximum retry delay (milliseconds). See
     * also {@link JoylinkMqttClient#getMaxRetryDelay()}.
     */
    public static final int CONNECTION_MAX_RETRY_DELAY = 30000;

    /**
     * The default value for clean session connections.
     * See also {@link JoylinkMqttClient#isCleanSession()}.
     */
    public static final boolean CLEAN_SESSION = true;

    /**
     * The default value for maximum offline queue size. See also
     * {@link JoylinkMqttClient#getMaxOfflineQueueSize()}.
     */
    public static final int MAX_OFFLINE_QUEUE_SIZE = 64;

    /**
     * The default value for enabling device update versioning. See also
     * {@link JoylinkDevice#isEnableVersioning()}.
     */
    public static final boolean DEVICE_ENABLE_VERSIONING = false;

    /**
     * The default value for device thing-model post QoS level. See also
     * {@link JoylinkDevice#getDeviceThingModelPostQos()}.
     */
    public static final int DEVICE_THING_MODEL_POST_QOS = 0;

    /**
     * The default value for the QoS level for subscribing to thing-model post response.
     * See also {@link JoylinkDevice#getDeviceThingModelPostResQos()}.
     */
    public static final int DEVICE_THING_MODEL_POST_RES_QOS = 0;

    /**
     * The default value for device properties post QoS level.
     * See also {@link JoylinkDevice#getDevicePropPostQos()}.
     */
    public static final int DEVICE_PROP_POST_QOS = 0;

    /**
     * The default value for the QoS level for subscribing to properties get.
     * See also {@link JoylinkDevice#getDevicePropGetQos()}.
     */
    public static final int DEVICE_PROP_GET_QOS = 0;

    /**
     * The default value for QoS level for publishing properties get response.
     * See also {@link JoylinkDevice#getDevicePropGetResQos()}.
     */
    public static final int DEVICE_PROP_GET_RES_QOS = 0;

    /**
     * The default value for QoS level for subscribing to properties set.
     * See also {@link JoylinkDevice#getDevicePropSetQos()}.
     */
    public static final int DEVICE_PROP_SET_QOS = 0;

    /**
     * The default value for QoS level for publishing properties set response.
     * See also {@link JoylinkDevice#getDevicePropSetResQos()}.
     */
    public static final int DEVICE_PROP_SET_RES_QOS = 0;

    /**
     * The default value for QoS level for subscribing to functions call.
     * See also {@link JoylinkDevice#getDeviceFuncCallQos()}.
     */
    public static final int DEVICE_FUNC_CALL_QOS = 0;

    /**
     * The default value for QoS level for publishing functions call response.
     * See also {@link JoylinkDevice#getDeviceFuncCallResQos()}.
     */
    public static final int DEVICE_FUNC_CALL_RES_QOS = 0;

    /**
     * The default value for device event post QoS level. See also
     * {@link JoylinkDevice#getDevicePropEventPostQos()}.
     */
    public static final int DEVICE_EVENT_POST_QOS = 0;

}
