/*
 * Copyright 2020 JD.com, Inc. or its affiliates. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License").
 * You may not use this file except in compliance with the License.
 * A copy of the License is located at
 *
 *  https://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * or in the "license" file accompanying this file. This file is distributed
 * on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

package com.joylink5.services.iot.client;

import lombok.Getter;
import lombok.Setter;

/**
 * This is a generic exception that can be thrown in most of the APIs, blocking
 * and non-blocking, by the library.
 */
public class JoylinkException extends Exception {

    /** The Constant serialVersionUID. */
    private static final long serialVersionUID = 1L;

    /**
     * Error code for joylink methods. It's only applicable to exceptions thrown
     * by those joylink method APIs.
     *
     * @param errorCode the new error code for the joylink method exception
     * @return the error code of the joylink method exception
     */
    @Getter
    @Setter
    private JoylinkDeviceErrorCode errorCode;

    /**
     * Instantiates a new exception object.
     *
     * @param message
     *            the error message
     */
    public JoylinkException(String message) {
        super(message);
    }

    /**
     * Instantiates a new exception object.
     *
     * @param errorCode
     *            the error code
     * @param message
     *            the error message
     */
    public JoylinkException(JoylinkDeviceErrorCode errorCode, String message) {
        super(message);
        this.errorCode = errorCode;
    }

    /**
     * Instantiates a new exception object.
     *
     * @param cause
     *            the cause. A null value is permitted, and indicates that the
     *            cause is nonexistent or unknown.
     */
    public JoylinkException(Throwable cause) {
        super(cause);
    }

}
