/*
 * Copyright 2020 JD.com, Inc. or its affiliates. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License").
 * You may not use this file except in compliance with the License.
 * A copy of the License is located at
 *
 *  https://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * or in the "license" file accompanying this file. This file is distributed
 * on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

package com.joylink5.services.iot.client;

import java.util.Map;
import java.util.ArrayList;
import java.util.List;
import java.util.HashMap;

import org.bouncycastle.pqc.math.linearalgebra.GoppaCode.MaMaPe;

import lombok.Getter;
import lombok.Setter;
/**
 *
 */
@Getter
@Setter
public class JoylinkOtaParameter {
    public enum OtaFunArgsName {
        TASK_ID("task-id"),
        PACKAGE_NAME("package-name"),
        NAME("name"),
        VERSION("version"),
        OBJECT_ID("object-id"),
        OBJECT_TYPE("object-type"),
        URL("url"),
        SIZE("size"),
        TOKEN("token"),
        SIGNATURE("signature"),
        DIFF_BASE_VERSION("diff-base-version"),
        DIFF_BASE_SIGNATURE("diff-base-signature"),
        DIFF_URL("diff-url"),
        DIFF_SIZE("diff-size"),
        DIFF_TOKEN("diff-token"),
        DIFF_SIGNATURE("diff-signature"),
        SIGNATURE_ALGORITHM("signature-algorithm"),
        INSTALL_MODE("install-mode"),
        TIME("time"),
        CUSTOMIZED("customized");

        private String argName;

        private OtaFunArgsName (final String argName) {
            this.argName = argName;
        }

        public String getValue() {
            return this.argName;
        }
    }

    // signature algorithm
    public enum OtaSigAlgorithm {
        MD5("0"),
        SHA256("1"),
        SHA256WithRSA("2"),
        SM3("3"),
        SM3WithSM2("4");

        private final String alg;

        private OtaSigAlgorithm (final String alg) {
            this.alg = alg;
        }

        public String getValue() {
            return this.alg;
        }
    }

    // ota object type
    public enum OtaObjectType {
        APP("app"),
        FIRMWARE("firmware"),
        FRAMWORK("framework"),
        OS("os");

        private final String objType;
        private OtaObjectType(String type) {
            this.objType = type;
        }
        public String getValue() {
            return this.objType;
        }

    }

    // ota intall mode
    public enum OtaInstallMode {
        IMMEDIATELY("0"),
        U_CFM_DOWN_INS("1"),
        U_CFM_INS("2"),
        ON_TIMING("3"),
        IDLE("4");

        private final String mode;
        private OtaInstallMode(String mode) {
            this.mode = mode;
        }
        public String getValue() {
            return this.mode;
        }
    }

    // ota state
    public enum OtaState {
        PREPARE("1"),
        WAIT_CONFORM_DOWNLOAD("2"),
        DOWNLOADING("3"),
        DOWNLOAD_DONE("4"),
        WAIT_CONFORM_INSTALL("5"),
        INSTALLING("6"),
        SUCCESS("99"),
        FAILED("-1");

        private final String state;
        private OtaState(String state) {
            this.state = state;
        }
        public String getValue() {
            return this.state;
        }
    }

    // error code
    public enum OtaErrorCode {
        NONE("0"),
        TASK_CANCEL("1001"),
        USER_CANCEL_DOWNLOAD("2001"),
        USER_CANCEL_INSTALL("2002"),
        DIFF_PAAGE_CHECK_FAILED("3001"),
        DOWNLOAD_FAILED("3002"),
        SIGNATURE_VERIFY_FAILED("3003"),
        DIFF_PAAGE_REVIV_FAILED("3004"),
        INSTALL_FAILED("3005");

        private final String errorCode;
        private OtaErrorCode(String code) {
            this.errorCode = code;
        }
        public String getValue() {
            return this.errorCode;
        }
    }
}
