/*
 * Copyright 2020 JD.com, Inc. or its affiliates. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License").
 * You may not use this file except in compliance with the License.
 * A copy of the License is located at
 *
 *  https://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * or in the "license" file accompanying this file. This file is distributed
 * on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

package com.joylink5.services.iot.client.core;

import java.security.KeyStore;
import javax.net.SocketFactory;
import javax.net.ssl.SSLSocketFactory;

import com.joylink5.services.iot.client.JoylinkException;
import com.joylink5.services.iot.client.mqtt.JoylinkMqttConnection;
import com.joylink5.services.iot.client.util.JoylinkTlsSocketFactory;

/**
 * This is a thin layer on top of {@link JoylinkMqttConnection} that provides a
 * TLS v1.2 based communication channel to the MQTT implementation.
 */
public class JoylinkTlsConnection extends JoylinkMqttConnection {

    public JoylinkTlsConnection(AbstractJoylinkClient client, KeyStore keyStore, String keyPassword)
            throws JoylinkException {
        super(client, new JoylinkTlsSocketFactory(keyStore, keyPassword), "ssl://" + client.getClientEndpoint() + ":" + client.getPort());
    }

    public JoylinkTlsConnection(AbstractJoylinkClient client, SSLSocketFactory socketFactory) throws JoylinkException {
        super(client, new JoylinkTlsSocketFactory(socketFactory), "ssl://" + client.getClientEndpoint() + ":" + client.getPort());
    }
    
}
