/*
 * Copyright 2020 JD.com, Inc. or its affiliates. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License").
 * You may not use this file except in compliance with the License.
 * A copy of the License is located at
 *
 *  https://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * or in the "license" file accompanying this file. This file is distributed
 * on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

package com.joylink5.services.iot.client.mqtt;

import org.eclipse.paho.client.mqttv3.IMqttDeliveryToken;
import org.eclipse.paho.client.mqttv3.MqttCallback;
import org.eclipse.paho.client.mqttv3.MqttMessage;

import com.joylink5.services.iot.client.JoylinkMessage;
import com.joylink5.services.iot.client.JoylinkQos;
import com.joylink5.services.iot.client.core.AbstractJoylinkClient;

/**
 * This class implements listener functions for client related events from the
 * Paho MQTT library.
 */
public class JoylinkMqttClientListener implements MqttCallback {

    private AbstractJoylinkClient client;

    public JoylinkMqttClientListener(AbstractJoylinkClient client) {
        this.client = client;
    }

    @Override
    public void connectionLost(Throwable arg0) {
        client.scheduleTask(new Runnable() {
            @Override
            public void run() {
                client.getConnection().onConnectionFailure();
            }
        });
    }

    @Override
    public void deliveryComplete(IMqttDeliveryToken arg0) {
        // Callback is not used
    }

    @Override
    public void messageArrived(String topic, MqttMessage arg1) throws Exception {
        JoylinkMessage message = new JoylinkMessage(topic, JoylinkQos.valueOf(arg1.getQos()), arg1.getPayload());
        client.dispatch(message);
    }

}
