/*
 * Copyright 2020 JD.com, Inc. or its affiliates. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License").
 * You may not use this file except in compliance with the License.
 * A copy of the License is located at
 *
 *  https://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * or in the "license" file accompanying this file. This file is distributed
 * on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

package com.joylink5.services.iot.client.protocol;

import java.util.logging.Logger;

import com.joylink5.services.iot.client.JoylinkDeviceErrorCode;
import com.joylink5.services.iot.client.JoylinkMessage;
import com.joylink5.services.iot.client.JoylinkQos;

public class JoylinkDeviceReportMessage extends JoylinkMessage {

    private static final Logger LOGGER = Logger.getLogger(JoylinkDeviceReportMessage.class.getName());

    private final AbstractJoylinkDevice device;
    private final long reportVersion;

    public JoylinkDeviceReportMessage(String topic, JoylinkQos qos, long reportVersion, String jsonState,
            AbstractJoylinkDevice device) {
        super(topic, qos, jsonState);
        this.device = device;
        this.reportVersion = reportVersion;
    }

    @Override
    public void onSuccess() {
        // increment local version only if it hasn't be updated
        device.getLocalVersion().compareAndSet(reportVersion, reportVersion + 1);
    }

    @Override
    public void onFailure() {
        if (JoylinkDeviceErrorCode.CONFLICT.equals(errorCode)) {
            LOGGER.warning("Device version conflict, restart version synchronization");
            // device.startVersionSync();
        } else {
            LOGGER.warning("Failed to publish device report: " + errorMessage);
        }
    }

}
