/*
 * Copyright 2020 JD.com, Inc. or its affiliates. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License").
 * You may not use this file except in compliance with the License.
 * A copy of the License is located at
 *
 *  https://www.apache.org/licenses/LICENSE-2.0.txt
 *
 * or in the "license" file accompanying this file. This file is distributed
 * on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either
 * express or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */

package com.joylink5.services.iot.client.protocol;

import java.io.IOException;
import java.util.Map;
import java.util.HashMap;
import java.util.logging.Logger;

import com.joylink5.services.iot.client.JoylinkMessage;
import com.joylink5.services.iot.client.JoylinkQos;
import com.joylink5.services.iot.client.JoylinkTopic;

import org.bouncycastle.crypto.CryptoServicesRegistrar.Property;

import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

import com.fasterxml.jackson.core.JsonGenerator;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.JsonSerializer;
import com.fasterxml.jackson.databind.SerializerProvider;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.JsonNode;

import com.joylink5.services.iot.client.core.AbstractJoylinkClient;

/**
 * This class extends {@link JoylinkTopic} to provide a callback function for
 * receiving the thing-model post response.
 */
public class JoylinkDeviceThingModelPostResponseListener extends JoylinkTopic {

    private static final Logger LOGGER = Logger.getLogger(JoylinkDeviceThingModelPostResponseListener.class.getName());

    // private final AbstractJoylinkDevice device;

    private AbstractJoylinkClient client;

    public JoylinkDeviceThingModelPostResponseListener(String topic, JoylinkQos qos, AbstractJoylinkClient client) {
        super(topic, qos);
        this.client = client;
    }

    @Override
    public void onMessage(JoylinkMessage message) {
        String payload = message.getStringPayload();
        LOGGER.info("\r\nRecv Message:" + message.getTopic() + "\r\n" + payload + "\r\n");
        if (payload == null) {
            LOGGER.warning("Received Empty Message for Topic " + message.getTopic());
            return;
        }
    }

    @Override
    public void onSuccess() {
    }

    @Override
    public void onFailure() {
        LOGGER.warning("Failed to subscribe to device topic " + topic);
    }

    @Override
    public void onTimeout() {
        LOGGER.warning("Timeout when subscribing to device topic " + topic);
    }

}
